/*
  Modul HTTP for ESP_32
  part of Arduino Mega Server project

*/

#ifdef HTTP_FEATURE

#include "WebServer.h"
#include "Update.h"

#define HTTP_PORT 8080
WebServer HTTP(HTTP_PORT);

File fsUploadFile;
unsigned long t_start,t_stop;

double calcSpeed(unsigned long ms, size_t len){
  return (double)(len * 125) / (double)(ms * 16);
}

String getContentType(String filename){
  if(HTTP.hasArg("download")) return "application/octet-stream";
  else if(filename.endsWith(".htm")) return "text/html";
  else if(filename.endsWith(".css")) return "text/css";
  else if(filename.endsWith(".csv")) return "text/plain";
  else if(filename.endsWith(".log")) return "text/plain";
  else if(filename.endsWith(".msg")) return "text/plain"; 
  else if(filename.endsWith(".cfg")) return "text/plain";
  else if(filename.endsWith(".js")) return "application/javascript";
  else if(filename.endsWith(".png")) return "image/png";
  else if(filename.endsWith(".gif")) return "image/gif";
  else if(filename.endsWith(".jpg")) return "image/jpeg";
  else if(filename.endsWith(".ico")) return "image/x-icon";
  else if(filename.endsWith(".xml")) return "text/xml";
  else if(filename.endsWith(".pdf")) return "application/x-pdf";
  else if(filename.endsWith(".zip")) return "application/x-zip";
  else if(filename.endsWith(".gz")) return "application/x-gzip";
  return "text/plain";
}

bool handleFileRead(String path){
  Serial.println("handleFileRead: " + path);
  if(path.endsWith("/")) path += "index.htm";
  String contentType = getContentType(path);
  String pathWithGz = path + ".gz";
  if(SPIFFS.exists(pathWithGz) || SPIFFS.exists(path)){
    if(SPIFFS.exists(pathWithGz)) 
      path += ".gz";
    File file = SPIFFS.open(path, "r");
    HTTP.streamFile(file, contentType);
    file.close();
    return true;
  }
  return false;
}

void handleFileUpload(){
  if(HTTP.uri() != "/edit") return;
  HTTPUpload& upload = HTTP.upload();
  if(upload.status == UPLOAD_FILE_START){
    String filename = upload.filename;
    if(!filename.startsWith("/")) filename = "/"+filename;
    fsUploadFile = SPIFFS.open(filename, "w");
    filename = String();
  } else if(upload.status == UPLOAD_FILE_WRITE){
    if(fsUploadFile)
      fsUploadFile.write(upload.buf, upload.currentSize);
  } else if(upload.status == UPLOAD_FILE_END){
    if(fsUploadFile)
      fsUploadFile.close();
  }
}

void handleFileDelete(){
  if(HTTP.args() == 0) return HTTP.send(500, "text/plain", "BAD ARGS");
  String path = HTTP.arg(0);
  if(path == "/")
    return HTTP.send(500, "text/plain", "BAD PATH");
  if(!SPIFFS.exists(path))
    return HTTP.send(404, "text/plain", "FileNotFound");
  SPIFFS.remove(path);
  HTTP.send(200, "text/plain", "");
  path = String();
}

void handleFileCreate(){
  if(HTTP.args() == 0)
    return HTTP.send(500, "text/plain", "BAD ARGS");
  String path = HTTP.arg(0);
  if(path == "/")
    return HTTP.send(500, "text/plain", "BAD PATH");
  if(SPIFFS.exists(path))
    return HTTP.send(500, "text/plain", "FILE EXISTS");
  File file = SPIFFS.open(path, "w");
  if(file)
    file.close();
  else
    return HTTP.send(500, "text/plain", "CREATE FAILED");
  HTTP.send(200, "text/plain", "");
  path = String();
}

void handleFileList() {
  if(!HTTP.hasArg("dir")) {HTTP.send(500, "text/plain", "BAD ARGS"); return;}
  String path = HTTP.arg("dir");
  if(path != "/" && !SPIFFS.exists((char *)path.c_str())) {HTTP.send(500, "text/plain", "BAD PATH"); return;}
  File dir = SPIFFS.open((char *)path.c_str());
  path = String();
  if(!dir.isDirectory()){
    dir.close();
    HTTP.send(500, "text/plain", "BAD DIR");
    return;
  }
  dir.rewindDirectory();
  HTTP.setContentLength(CONTENT_LENGTH_UNKNOWN);
  HTTP.send(200, "text/json", "");

  HTTP.sendContent("[");
  spisok = ""; // удаляем если список был
  for (int cnt = 0; true; ++cnt) {
    File entry = dir.openNextFile();
    if (!entry)
    break;
    String output;
    if (cnt > 0) { output = ','; }
    output += "{\"type\":\"";
    output += (entry.isDirectory()) ? "dir" : "file";
    output += "\",\"name\":\"";
    output += String(entry.name()).substring(1);
    spisok += " ";
    spisok += String(entry.name()).substring(1);
    spisok += ":";
    spisok += String(entry.size());
    spisok += ",";
    output += "\"";
    output += "}";
    HTTP.sendContent(output);
    entry.close();
    //Serial.println();
    //Serial.println(output);
    //Serial.println();
 }
 HTTP.sendContent("]");
 dir.close();
 //Serial.println();
 //Serial.println(spisok);
 //Serial.println();
}

String Fail = "<!DOCTYPE html><html><head><meta http-equiv='Content-type' content='text/html; charset=utf-8'><meta name='viewport' content='width=device-width, initial-scale=1'></head><style>body{background:#FFA07A;}</style><body><h3>Firmware update:</h3><h2>&emsp;&ensp;FAIL</h2><h3>&ensp;&ensp;Restart ESP</h3></body></html>";

String Ok = "<!DOCTYPE html><html><head><meta http-equiv='Content-type' content='text/html; charset=utf-8'><meta name='viewport' content='width=device-width, initial-scale=1'><meta http-equiv='refresh' content='15;update.htm'></head><style>body{background:#E0EEE0;}</style><body><h3>Firmware update:</h3><h2>&emsp;&ensp;OK!</h2></body></html>";
void updateInit() {
  HTTP.on("/update", HTTP_POST, [](){
  HTTP.sendHeader("Connection", "close");
  HTTP.send(200, "text/html", (Update.hasError()) ? Fail : Ok);
  delay(500);//=================================
  ESP.restart();
  },[](){
    HTTPUpload& upload = HTTP.upload();
    if(upload.status == UPLOAD_FILE_START){
      t_start = millis();
      if (!Update.begin(UPDATE_SIZE_UNKNOWN)) {
        Update.printError(Serial);
      }
    } else if(upload.status == UPLOAD_FILE_WRITE){
      if(Update.write(upload.buf, upload.currentSize) != upload.currentSize){
        Update.printError(Serial);
      }
    } else if(upload.status == UPLOAD_FILE_END){
      if(Update.end(true)){
        t_stop = millis();
        Serial.print("Time UPLOAD: "); Serial.print((t_stop - t_start) / 1000.0); Serial.println(" sec.");
        Serial.print("Speed UPLOAD: "); Serial.print(calcSpeed(t_stop - t_start, upload.totalSize)); Serial.println(" Kbit/s");
        Serial.printf("Upload Success: %u bytes\n", upload.totalSize);
      } else {
        Update.printError(Serial);
      }
    }
    yield();
  });
} // updateInit()

void initHttp() {
  HTTP.on("/list", HTTP_GET, handleFileList);
  HTTP.on("/edit", HTTP_GET, []() { if (!handleFileRead("edit.htm")) HTTP.send(404, "text/plain", "FileNotFound"); });
  HTTP.on("/edit", HTTP_PUT, handleFileCreate);
  HTTP.on("/edit", HTTP_DELETE, handleFileDelete);
  HTTP.on("/edit", HTTP_POST, []() { HTTP.send(200, "text/plain", ""); }, handleFileUpload);
  HTTP.onNotFound([]() { if (!handleFileRead(HTTP.uri())) HTTP.send(404, "text/plain", "FileNotFound"); });

  updateInit();
  HTTP.begin();
  timeStamp();
  printValue("HTTP port", String(HTTP_PORT));
  moduleHTTP = ENABLE;
  started(F("Http"), true);
} // initHttp()

void httpWork() {
  HTTP.handleClient();
}

#endif // HTTP_FEATURE 
